#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <omp.h>

int *arr_gen(int size) {
	int *arr = malloc(size * sizeof(int));
	for (int i = 0; i < size; ++i) {
		arr[i] = rand() % (size + 1);
	}
	return arr;
}
int main(int argc, char **argv) {
	srand(time(NULL));
	int arr_size;
	if (argc == 3) {
		arr_size = atoi(argv[1]);
	}
	else {
		printf("Harus ada argumen jumlah array");
	}

	int num_threads = atoi(argv[2]);
	printf("%d\n", num_threads);

	int *array = arr_gen(arr_size);

#ifdef DEBUG
	printf("Original Array:\n");
	for (int i = 0; i < arr_size; ++i) {
		if (i) printf(" ");
		printf("%d", array[i]);
	}
	printf("\n");
#endif

	int max_val = arr_size;
	int range = max_val/num_threads+1;
	int **bucket = malloc(num_threads * sizeof(int*));

	struct timespec start_time, end_time;
	clock_gettime(CLOCK_MONOTONIC, &start_time);
	
	int *q = malloc(num_threads * sizeof(int));
	int *head = malloc(num_threads * sizeof(int));
	int i;
	for (i = 0; i < num_threads; ++i) {
		q[i] = 0;
		head[i] = 0;
	}
	for (i = 0; i < arr_size; ++i) {
		int index = array[i] / range;
		q[index]++;
	}
	for (i = 0; i < num_threads; ++i) {
		bucket[i] = malloc(q[i] * sizeof(int));
	}
	for (i = 0; i < arr_size; ++i) {
		int index = array[i] / range;
		bucket[index][head[index]] = array[i];
		head[index]++;
	}

	int thread_rank = omp_get_thread_num();
#pragma omp parallel num_threads(num_threads)
	{
		int i, j;
		int n = q[thread_rank];
		int *local_bucket = bucket[thread_rank];
		for (i = 0; i < n; ++i) {
			for (j = i+1; j < n; ++j) {
				if (local_bucket[i] > local_bucket[j]) {
					int c = local_bucket[j];
					local_bucket[j] = local_bucket[i];
					local_bucket[i] = c;
				}
			}
		}
	}

	int j;
	int p = 0;
	for (i = 0; i < num_threads; ++i) {
		for (j = 0; j < q[i]; ++j) {
			array[p++] = bucket[i][j];
		}
	}

	clock_gettime(CLOCK_MONOTONIC, &end_time);

#ifdef DEBUG
	printf("Hasil sorting:\n");
	for (i = 0; i < arr_size; ++i) {
		if (i) printf(" ");
		printf("%d", array[i]);
	}
	printf("\n");
#endif

	double diff = (end_time.tv_sec - start_time.tv_sec) + (double)(end_time.tv_nsec - start_time.tv_nsec) / 1000000000.0;
	printf("Program berjalan selama: %lf detik\n", diff);

	return 0;
}
